@@ -22,9 +22,12 @@ import contacts as ContactH
 import fmms_config as fMMSconf
 
 import logging
-
 log = logging.getLogger('fmms.%s' % __name__)
 
+_ = gettext.gettext
+gettext.bindtextdomain('fmms','/opt/fmms/share/locale/')
+gettext.textdomain('fmms')
+
 class fMMS_GUI(hildon.Program):
 	""" GUI class for the application. """
 
@@ -41,6 +44,7 @@ class fMMS_GUI(hildon.Program):
 		self.refreshlistview = True
 		self.viewerimported = False
 		self.senderimported = False
+		self._screenwidth = 800
 		
 		self.avatarlist = {}
 		self.namelist = {}
@@ -50,48 +54,30 @@ class fMMS_GUI(hildon.Program):
 		program = hildon.Program.get_instance()
 			
 		self.window = hildon.StackableWindow()
+		hildon.hildon_gtk_window_set_portrait_flags(self.window, hildon.PORTRAIT_MODE_SUPPORT)
 		gtk.set_application_name("fMMS")
 		self.window.set_title("fMMS")
 		program.add_window(self.window)
-		
 		self.window.connect("delete_event", self.quit)
 		
-		pan = hildon.PannableArea()
-		pan.set_property("mov-mode", hildon.MOVEMENT_MODE_VERT)
+		self.pan = hildon.PannableArea()
+		self.pan.set_property("mov-mode", hildon.MOVEMENT_MODE_VERT)
 		
-		### TODO: dont hardcode the values here.. oh well
-		iconcell = gtk.CellRendererPixbuf()
-		photocell = gtk.CellRendererPixbuf()
-		textcell = gtk.CellRendererText()
-		photocell.set_property('xalign', 1.0)
-		textcell.set_property('mode', gtk.CELL_RENDERER_MODE_INERT)
-		textcell.set_property('xalign', 0.0)
-		
-		self.liststore = gtk.ListStore(gtk.gdk.Pixbuf, str, gtk.gdk.Pixbuf, str, str)
+		# wonder how much memory this is wasting
+		self.iconcell = gtk.CellRendererPixbuf()
+		self.photocell = gtk.CellRendererPixbuf()
+		self.textcell = gtk.CellRendererText()
+		self.photocell.set_property('xalign', 1.0)
+		self.textcell.set_property('mode', gtk.CELL_RENDERER_MODE_INERT)
+		self.textcell.set_property('xalign', 0.0)
+
+		self.liststore = gtk.ListStore(gtk.gdk.Pixbuf, str, gtk.gdk.Pixbuf, str, str, str)
 		self.treeview = hildon.GtkTreeView(gtk.HILDON_UI_MODE_NORMAL)
 		self.treeview.set_property("fixed-height-mode", True)
 		self.treeview.set_model(self.liststore)
-
-		icon_col = gtk.TreeViewColumn('Icon')
-		sender_col = gtk.TreeViewColumn('Sender')
-		placeholder_col = gtk.TreeViewColumn('Photo')
-
-		icon_col.pack_start(iconcell, False)
-		icon_col.set_attributes(iconcell, pixbuf=0)
-		icon_col.set_fixed_width(64)
-		icon_col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-		sender_col.pack_start(textcell, True)
-		sender_col.set_attributes(textcell, markup=1)
-		sender_col.set_fixed_width(640)
-		sender_col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-		placeholder_col.pack_end(photocell, False)
-		placeholder_col.set_attributes(photocell, pixbuf=2)
-		placeholder_col.set_fixed_width(64)
-		placeholder_col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 		
-		self.treeview.append_column(icon_col)
-		self.treeview.append_column(sender_col)
-		self.treeview.append_column(placeholder_col)
+		# create ui
+		self._create_ui()
 
 		self.treeview.tap_and_hold_setup(self.liststore_mms_menu())
 		self.treeview.tap_and_hold_setup(None)
@@ -106,11 +92,11 @@ class fMMS_GUI(hildon.Program):
 		envelopeImage.set_alignment(1, 0.5)
 		mmsLabel = gtk.Label(gettext.ldgettext('rtcom-messaging-ui', "messaging_ti_new_mms"))
 		mmsLabel.set_alignment(0, 0.5)
-		
+
 		mmsBox.pack_start(envelopeImage, True, True, 0)
 		mmsBox.pack_start(mmsLabel, True, True, 0)
 		newMsgButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
-		
+
 		newMsgButton.add(mmsBox)
 		newMsgButton.connect('clicked', self.new_mms_button_clicked)
 
@@ -118,22 +104,76 @@ class fMMS_GUI(hildon.Program):
 		actionbox = self.treeview.get_action_area_box()
 		self.treeview.set_action_area_visible(True)
 		actionbox.add(newMsgButton)
-		
-		pan.add(self.treeview)
+
+		self.pan.add(self.treeview)
+
+		self.livefilter = hildon.LiveSearch()
+		modelfilter = self.liststore.filter_new()
+		modelfilter.set_visible_func(self.cb_filter_row)
+		self.treeview.set_model(modelfilter)
+		self.livefilter.set_filter(modelfilter)
+		self.livefilter.widget_hook(self.window, self.treeview)
+
+		contBox = gtk.VBox()
+		contBox.pack_start(self.pan, True, True, 0)
+		contBox.pack_start(self.livefilter, False, False, 0)
 
 		align = gtk.Alignment(1, 1, 1, 1)
 		align.set_padding(2, 2, 10, 10)		
-		align.add(pan)
-		
+		align.add(contBox)
 		self.window.add(align)
-	
-		self.menu = self.cont.create_menu(self.window)
-		self.window.set_app_menu(self.menu)
-		
+
+		menu = self.cont.create_menu(self.window)
+		self.window.set_app_menu(menu)
 		self.window.connect('focus-in-event', self.cb_on_focus)
-		
+		self.window.connect('configure-event', self._onOrientationChange)
 		self.window.show_all()
 		self.add_window(self.window)
+	
+	def _onOrientationChange(self, window, event):
+		newwidth = window.get_screen().get_width()
+		if newwidth == self._screenwidth:
+			return
+		else:
+			self._screenwidth = newwidth
+		self._create_ui()
+
+	def _create_ui(self):
+		for col in self.treeview.get_columns():
+			self.treeview.remove_column(col)
+	
+		icon_col = gtk.TreeViewColumn('Icon')
+		placeholder_col = gtk.TreeViewColumn('Photo')
+		self.sender_col = gtk.TreeViewColumn('Sender')
+
+		icon_col.pack_start(self.iconcell, False)
+		icon_col.set_attributes(self.iconcell, pixbuf=0)
+		icon_col.set_fixed_width(64)
+		icon_col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+
+		self.sender_col.pack_start(self.textcell, True)
+		self.sender_col.set_attributes(self.textcell, markup=1)
+		# this is kinda ugly
+		self.sender_col.set_fixed_width(self._screenwidth - 64 - 64 - 32)
+		self.sender_col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+
+		placeholder_col.pack_end(self.photocell, False)
+		placeholder_col.set_attributes(self.photocell, pixbuf=2)
+		placeholder_col.set_fixed_width(64)
+		placeholder_col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+
+		self.treeview.append_column(icon_col)
+		self.treeview.append_column(self.sender_col)
+		self.treeview.append_column(placeholder_col)
+
+	def cb_filter_row(self, model, iter, user_data=None):
+		txt = str(model.get_value(iter, 4)).lower()
+		desc = str(model.get_value(iter, 5)).lower()
+		filter = str(self.livefilter.get_text()).lower()
+		if filter in txt or filter in desc:
+			return True
+		else:
+			return False
 
 	def import_viewer(self):
 		""" This is used to import viewer only when we need it
@@ -184,6 +224,10 @@ class fMMS_GUI(hildon.Program):
 			t2 = time.clock()
 			log.info("liststore time: %s" % round(t2-t1, 3))
 			self.refreshlistview = False
+			modelfilter = self.liststore.filter_new()
+			modelfilter.set_visible_func(self.cb_filter_row)
+			self.treeview.set_model(modelfilter)
+			self.livefilter.set_filter(modelfilter)
 			
 			if self.config.get_firstlaunch() < 2:
 				settings = self.config.get_apn_settings()
@@ -200,7 +244,7 @@ class fMMS_GUI(hildon.Program):
 
 			self.take_ss()
 
-		return True
+		return False
 
 
 	def cb_open_fmms(self, interface, method, args, user_data):
@@ -221,7 +265,7 @@ class fMMS_GUI(hildon.Program):
 			return
 		elif method == 'send_mms':
 			log.info("launching sender with args: %s", args)
-			self.refreshlistview = True
+			self.refreshlistview = False
 			self.import_sender()
 			fMMSSenderUI.fMMS_SenderUI(tonumber=args[0]).run()
 			return
@@ -312,7 +356,7 @@ class fMMS_GUI(hildon.Program):
 			except:
 				description = varlist.get('Subject', '')
 
-			description = description.encode('utf-8', 'ignore')
+			description = description.decode('utf-8', 'ignore')
 			primarytext = ' <span font_desc="%s" foreground="%s"><sup>%s</sup></span>' % (secondarytxt, secondarycolor, mtime)
 			secondarytext = '\n<span font_desc="%s" foreground="%s">%s</span>' % (secondarytxt, secondarycolor, escape(description))
 			if not isread and direction == fMMSController.MSG_DIRECTION_IN:
@@ -320,7 +364,7 @@ class fMMS_GUI(hildon.Program):
 			else:
 				sender = escape(sender)
 			stringline = "%s%s%s" % (sender, primarytext, secondarytext)
-			self.liststore.append([icon, stringline, avatar, fname, sender])
+			self.liststore.append([icon, stringline, avatar, fname, sender, description])
 
 	def quit(self, *args):
 		""" Quits the application. """
@@ -356,7 +400,7 @@ class fMMS_GUI(hildon.Program):
 		if self.curPath == None:
 			return
 			
-		model = self.treeview.get_model()
+		model = self.treeview.get_model().get_model()
 		miter = model.get_iter(self.curPath)
 		# the 4th value is the transactionid (start counting at 0)
 		filename = model.get_value(miter, 3)
@@ -364,6 +408,7 @@ class fMMS_GUI(hildon.Program):
 		confirmtxt = gettext.ldgettext('rtcom-messaging-ui', "messaging_fi_delete_1_sms")
 		
 		dialog = gtk.Dialog()
+		dialog.set_transient_for(self.window)
 		dialog.set_title(confirmtxt)
 		dialog.add_button(gtk.STOCK_YES, 1)
 		dialog.add_button(gtk.STOCK_NO, 0)
@@ -374,8 +419,11 @@ class fMMS_GUI(hildon.Program):
 		if ret == 1:
 			log.info("Deleting %s", filename)
 			self.delete_push_mms(filename)
-			self.liststore.remove(miter)
+			model.remove(miter)
 		dialog.destroy()
+		hildon.hildon_gtk_window_take_screenshot(self.window, False)
+		hildon.hildon_gtk_window_take_screenshot(self.window, True)
+		self.force_ui_update()
 		return
 	
 	def liststore_mms_menu(self):
@@ -419,6 +467,7 @@ class fMMS_GUI(hildon.Program):
 			fMMSViewer.fMMS_Viewer(transactionid, spawner=self)
 		except Exception, e:
 			log.exception("Failed to open viewer with transaction id: %s" % transactionid)
+			hildon.hildon_banner_show_information(self.window, "", gettext.ldgettext('hildon-common-strings', "sfil_ni_operation_failed"))
 			#raise
 		hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
 		self.treeview.set_sensitive(True)
@@ -429,7 +478,7 @@ class fMMS_GUI(hildon.Program):
 		dialog = gtk.Dialog()
 		dialog.set_title(gettext.ldgettext('osso-connectivity-ui', 'conn_mngr_me_int_conn_change_iap'))
 		dialog.set_transient_for(self.window)
-		label = gtk.Label("Would you like me to terminate your current connection to fetch the MMS?")
+		label = gtk.Label(_("To retrieve the MMS your active connection will need to change. Switch connection?"))
 		label.set_line_wrap(True)
 		dialog.vbox.add(label)
 		dialog.add_button(gtk.STOCK_YES, 1)
